package org.ow2.petals.se.sca.binding;

import java.util.ArrayList;
import org.eclipse.stp.sca.BaseReference;
import org.eclipse.stp.sca.BaseService;
import org.eclipse.stp.sca.Binding;
import org.eclipse.stp.sca.Component;
import javax.jbi.component.ComponentContext;
import org.eclipse.stp.sca.ComponentReference;
import org.eclipse.stp.sca.ComponentService;
import org.eclipse.stp.sca.Composite;
import javax.xml.transform.dom.DOMSource;
import org.ow2.petals.databinding.jaxb.DataBindingException;
import org.w3c.dom.Document;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.common.util.EList;
import java.io.File;
import org.eclipse.stp.sca.domainmodel.frascati.FrascatiPackage;
import org.eclipse.stp.sca.Interface;
import org.eclipse.stp.sca.domainmodel.frascati.JBIBinding;
import javax.jbi.JBIException;
import org.eclipse.stp.sca.JavaInterface;
import java.util.List;
import java.util.Map;
import org.w3c.dom.Node;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import java.util.Properties;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import javax.xml.namespace.QName;
import org.eclipse.stp.sca.Reference;
import javax.xml.transform.Result;
import org.eclipse.stp.sca.ScaPackage;
import org.ow2.petals.databinding.jaxb.service.Service;
import javax.jbi.servicedesc.ServiceEndpoint;
import org.ow2.petals.databinding.jaxb.ServiceFactory;
import javax.xml.transform.Source;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import java.net.URL;

/** 
 * This class permits to process the JIB binding for the composite 
 * services/references to create the service databinding needed 
 * to marshll/unmarshall the message request/response. 
 * 
 * @author Mohammed EL JAI
 */
public class BindingProcess {
    /** 
     * Dispatch call to process all JBI binding declared in the SCA composite file.
     *  
     * @param emfComposite
     * @param scaComposite
     * @param provide
     * @param suRootPath
     * @param componentContext
     * @param wsdlFileName
     * @throws PEtALSCDKException
     */
    public static void processBinding(Composite emfComposite, ClassLoader compositeClassLoader, Provides provide, String suRootPath, ComponentContext componentContext, String wsdlFileName, URL[] seClassPath, Map<org.ow2.petals.component.framework.jbidescriptor.generated.Provides, org.ow2.petals.databinding.jaxb.service.Service>  dataBindingServicePool, List<javax.xml.namespace.QName>  listOfReference) throws PEtALSCDKException {
        BindingProcess.processCompositeServices(emfComposite ,compositeClassLoader ,provide ,suRootPath ,wsdlFileName ,seClassPath ,dataBindingServicePool);
        BindingProcess.processComponentServices(emfComposite ,compositeClassLoader ,provide ,suRootPath ,wsdlFileName ,seClassPath ,dataBindingServicePool);
        BindingProcess.processCompositeReferences(emfComposite ,componentContext ,compositeClassLoader ,suRootPath ,seClassPath ,listOfReference);
        BindingProcess.processComponentReferences(emfComposite ,componentContext ,compositeClassLoader ,suRootPath ,seClassPath ,listOfReference);
    }
    
    /** 
     * Create the databinding service for the remote components references (JBI binding).
     * 
     * @param emfComposite
     * @param componentContext
     * @param scaComposite
     * @param suRootPath
     * @throws PEtALSCDKException
     */
    private static void processComponentReferences(Composite emfComposite, ComponentContext componentContext, ClassLoader compositeClassLoader, String suRootPath, URL[] seClassPath, List<javax.xml.namespace.QName>  listOfReference) throws PEtALSCDKException {
        EList<org.eclipse.stp.sca.Component>  componentList = emfComposite.getComponent();
        for (Component component : componentList) {
            EList<org.eclipse.stp.sca.ComponentReference>  referencies = component.getReference();
            for (ComponentReference componentReference : referencies) {
                EList<org.eclipse.stp.sca.Binding>  serviceBindings = componentReference.getBinding();
                if ((serviceBindings.size()) > 0) {
                    for (int i = 0 ; i < (componentReference.getBinding().size()) ; i++) {
                        Binding bd = ((Binding)(componentReference.getBinding().get(i)));
                        switch (bd.eClass().getClassifierID()) {
                            case ScaPackage.SCA_BINDING :
                                break;
                            case FrascatiPackage.JBI_BINDING :
                                JBIBinding currentBinding = ((JBIBinding)(bd));
                                BindingProcess.processJbiReference(currentBinding ,componentContext ,suRootPath ,componentReference ,compositeClassLoader ,seClassPath ,listOfReference);
                                break;
                            default :
                        }
                    }
                } 
            }
        }
    }
    
    /** 
     *  Create the databinding service for the remote composite references (JBI binding).
     *  
     * @param emfComposite
     * @param componentContext
     * @param scaComposite
     * @param suRootPath
     * @throws PEtALSCDKException
     */
    private static void processCompositeReferences(Composite emfComposite, ComponentContext componentContext, ClassLoader compositeClassLoader, String suRootPath, URL[] seClassPath, List<javax.xml.namespace.QName>  listOfReference) throws PEtALSCDKException {
        EList<org.eclipse.stp.sca.Reference>  references = emfComposite.getReference();
        for (Reference reference : references) {
            EList<org.eclipse.stp.sca.Binding>  refernceBindings = reference.getBinding();
            if ((refernceBindings.size()) > 0) {
                Binding bd = ((Binding)(reference.getBinding().get(0)));
                switch (bd.eClass().getClassifierID()) {
                    case ScaPackage.SCA_BINDING :
                        break;
                    case FrascatiPackage.JBI_BINDING :
                        JBIBinding currentBinding = ((JBIBinding)(bd));
                        if (!((((reference.getPromote2()) != null) && ((reference.getPromote2().getBinding().size()) > 0)) && ((reference.getPromote2().getBinding().get(0)) instanceof JBIBinding)))
                            BindingProcess.processJbiReference(currentBinding ,componentContext ,suRootPath ,reference ,compositeClassLoader ,seClassPath ,listOfReference);
                        
                        break;
                    default :
                }
            } 
        }
    }
    
    /** 
     * Create the databinding service for the remote component services (JBI binding).
     * 
     * @param emfComposite
     * @param scaComposite
     * @param provides
     * @param rootPath
     * @throws PEtALSCDKException
     */
    private static void processComponentServices(Composite emfComposite, ClassLoader compositeClassLoader, Provides provides, String rootPath, String wsdlFileName, URL[] seClassPath, Map<org.ow2.petals.component.framework.jbidescriptor.generated.Provides, org.ow2.petals.databinding.jaxb.service.Service>  dataBindingServicePool) throws PEtALSCDKException {
        EList<org.eclipse.stp.sca.Component>  componentList = emfComposite.getComponent();
        for (Component component : componentList) {
            EList<org.eclipse.stp.sca.ComponentService>  services = component.getService();
            for (ComponentService componentService : services) {
                EList<org.eclipse.stp.sca.Binding>  serviceBindings = componentService.getBinding();
                if ((serviceBindings.size()) > 0) {
                    Binding bd = ((Binding)(componentService.getBinding().get(0)));
                    switch (bd.eClass().getClassifierID()) {
                        case ScaPackage.SCA_BINDING :
                            break;
                        case FrascatiPackage.JBI_BINDING :
                            JBIBinding currentBinding = ((JBIBinding)(bd));
                            BindingProcess.processJbiService(currentBinding ,rootPath ,componentService ,compositeClassLoader ,provides ,wsdlFileName ,seClassPath ,dataBindingServicePool);
                            break;
                        default :
                    }
                } 
            }
        }
    }
    
    /** 
     * Create the databinding service for the remote composite services (JBI binding).
     * 
     * @param emfComposite
     * @param scaComposite
     * @param provides
     * @param rootPath
     * @throws PEtALSCDKException
     */
    private static void processCompositeServices(Composite emfComposite, ClassLoader compositeClassLoader, Provides provides, String rootPath, String wsdlFileName, URL[] seClassPath, Map<org.ow2.petals.component.framework.jbidescriptor.generated.Provides, org.ow2.petals.databinding.jaxb.service.Service>  dataBindingServicePool) throws PEtALSCDKException {
        EList<org.eclipse.stp.sca.Service>  services = emfComposite.getService();
        for (org.eclipse.stp.sca.Service service : services) {
            EList<org.eclipse.stp.sca.Binding>  serviceBindings = service.getBinding();
            if ((serviceBindings.size()) > 0) {
                Binding bd = ((Binding)(service.getBinding().get(0)));
                switch (bd.eClass().getClassifierID()) {
                    case ScaPackage.SCA_BINDING :
                        break;
                    default :
                    case FrascatiPackage.JBI_BINDING :
                        JBIBinding currentBinding = ((JBIBinding)(bd));
                        BindingProcess.processJbiService(currentBinding ,rootPath ,service ,compositeClassLoader ,provides ,wsdlFileName ,seClassPath ,dataBindingServicePool);
                        break;
                }
            } 
        }
    }
    
    /** 
     * 
     * 
     * @param currentBinding
     * @param rootPath
     * @param componentService
     * @param scaComposite
     * @throws PEtALSCDKException 
     */
    private static void processJbiService(JBIBinding jbiBinding, String rootPath, BaseService service, ClassLoader compositeClassLoader, Provides provides, String wsdlFileName, URL[] seClassPath, Map<org.ow2.petals.component.framework.jbidescriptor.generated.Provides, org.ow2.petals.databinding.jaxb.service.Service>  dataBindingServicePool) throws PEtALSCDKException {
        Properties saveProperties = ((Properties)(java.lang.System.getProperties().clone()));
        String wsdlFilePath = (rootPath + (File.separator)) + wsdlFileName;
        String jaxbBaseDir = (rootPath + "/jaxb-service-") + (service.getName());
        new File(jaxbBaseDir).mkdir();
        org.ow2.petals.databinding.jaxb.tools.JaxbClassGenerator.generateJaxbClass(wsdlFilePath ,jaxbBaseDir ,service.getName());
        List<java.lang.String>  classPath = new ArrayList<java.lang.String> ();
        for (URL path : seClassPath)
            classPath.add(path.getPath());
        org.ow2.petals.databinding.jaxb.tools.JaxbClassGenerator.compileJaxbClass(jaxbBaseDir ,service.getName() ,classPath);
        String jaxbClassesUrl = ("file://" + jaxbBaseDir) + "/target/classes/";
        ServiceFactory serviceFactory = ServiceFactory.getInstance();
        try {
            Service dataBindingService = serviceFactory.crateService(wsdlFilePath ,jaxbClassesUrl ,compositeClassLoader ,compositeClassLoader.loadClass(((JavaInterface)(service.getInterface())).getInterface()));
            dataBindingServicePool.put(provides ,dataBindingService);
        } catch (DataBindingException e) {
            throw new PEtALSCDKException(("can\'t create the service databinding for the " + "JBI binding") , e);
        } catch (ClassNotFoundException e) {
            throw new PEtALSCDKException(("can\'t load the java interface for the " + "composite service ") , e);
        } finally {
            java.lang.System.setProperties(saveProperties);
            Runtime.getRuntime().gc();
        }
    }
    
    /** 
     * 
     * @param jbiBinding
     * @param componentContext
     * @param suRootPath
     * @param reference
     * @param compositeClassLoader
     * @return
     * @throws PEtALSCDKException
     */
    private static void processJbiReference(JBIBinding jbiBinding, ComponentContext componentContext, String suRootPath, BaseReference reference, ClassLoader compositeClassLoader, URL[] seClassPath, List<javax.xml.namespace.QName>  listOfReference) throws PEtALSCDKException {
        ServiceEndpoint serviceEndpoint = null;
        Properties saveProperties = ((Properties)(java.lang.System.getProperties().clone()));
        try {
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = null;
            transformer = tFactory.newTransformer();
            QName serviceName = new QName(jbiBinding.getServiceNamespace() , jbiBinding.getServiceName());
            String wsdlFilePath = null;
            if ((jbiBinding.getWsdl()) != null) {
                wsdlFilePath = suRootPath + (jbiBinding.getWsdl());
            } else {
                ServiceEndpoint[] svcEndpoints = componentContext.getEndpointsForService(serviceName);
                if ((svcEndpoints.length) > 0) {
                    serviceEndpoint = svcEndpoints[0];
                    Document wsdlDoc = null;
                    wsdlDoc = componentContext.getEndpointDescriptor(serviceEndpoint);
                    DOMSource source = new DOMSource(wsdlDoc);
                    wsdlFilePath = (suRootPath + (reference.getName())) + ".wsdl";
                    File wsdlFile = new File(wsdlFilePath);
                    StreamResult result = new StreamResult(wsdlFile);
                    transformer.transform(source ,result);
                } 
            }
            String jaxbBaseDir = (suRootPath + "/jaxb-service-") + (reference.getName());
            new File(jaxbBaseDir).mkdir();
            org.ow2.petals.databinding.jaxb.tools.JaxbClassGenerator.generateJaxbClass(wsdlFilePath ,jaxbBaseDir ,reference.getName());
            List<java.lang.String>  classPath = new ArrayList<java.lang.String> ();
            for (URL path : seClassPath)
                classPath.add(path.getPath());
            org.ow2.petals.databinding.jaxb.tools.JaxbClassGenerator.compileJaxbClass(jaxbBaseDir ,reference.getName() ,classPath);
            String jaxbClassesUrl = ("file://" + jaxbBaseDir) + "/target/classes/";
            ServiceFactory serviceFactory = ServiceFactory.getInstance();
            Service service = serviceFactory.crateService(wsdlFilePath ,jaxbClassesUrl ,compositeClassLoader ,compositeClassLoader.loadClass(((JavaInterface)(reference.getInterface())).getInterface()));
            org.ow2.petals.se.sca.PetalsGateway.getSuReferencesPool().put(serviceName ,service);
            listOfReference.add(serviceName);
        } catch (TransformerConfigurationException e) {
            throw new PEtALSCDKException(("can\'t create Transformer to parse the " + "JBI binding") , e);
        } catch (JBIException e) {
            throw new PEtALSCDKException(("can\'t get the WSDL for the " + "JBI binding ") , e);
        } catch (TransformerException e) {
            throw new PEtALSCDKException(("can\'t import the WSDL for the " + "JBI binding") , e);
        } catch (DataBindingException e) {
            throw new PEtALSCDKException(("can\'t create the service databinding for the " + "JBI binding") , e);
        } catch (ClassNotFoundException e) {
            throw new PEtALSCDKException(("can\'t load the java interface for the " + "composite reference ") , e);
        } finally {
            java.lang.System.setProperties(saveProperties);
            Runtime.getRuntime().gc();
        }
    }
    
}

