package org.ow2.petals.se.sca.connectors.jbi;

import javax.jbi.component.ComponentContext;
import javax.xml.transform.dom.DOMSource;
import org.ow2.petals.databinding.jaxb.DataBindingException;
import javax.jbi.messaging.DeliveryChannel;
import org.w3c.dom.Document;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.InOnly;
import javax.jbi.messaging.InOut;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.MessageExchangeFactory;
import java.lang.reflect.Method;
import org.w3c.dom.Node;
import javax.jbi.messaging.NormalizedMessage;
import org.ow2.petals.databinding.jaxb.service.Operation;
import javax.xml.namespace.QName;
import org.ow2.petals.databinding.jaxb.service.Service;
import javax.jbi.servicedesc.ServiceEndpoint;
import org.ow2.petals.databinding.jaxb.serialization.ServiceMarshaller;
import org.ow2.petals.databinding.jaxb.serialization.ServiceUnmarshaller;
import javax.xml.transform.Source;

/** 
 * 
 * @author Mohammed EL JAI
 */
public class JbiClient {
    /** 
     *
     */
    private DeliveryChannel deliveryChannel;
    
    /** 
     *
     */
    private ComponentContext componentContext;
    
    /** 
     *
     */
    private QName serviceName;
    
    /** 
     *
     */
    private Service service;
    
    /** 
     * 
     * @param deliveryChannel
     * @param componentContext
     * @param jbiService
     * @param serviceName
     */
    public JbiClient(DeliveryChannel deliveryChannel ,ComponentContext componentContext ,Service service ,QName serviceName) {
        super();
        this.serviceName = serviceName;
        this.deliveryChannel = deliveryChannel;
        this.componentContext = componentContext;
        this.service = service;
    }
    
    /** 
     * 
     *   
     * @param method : The service method to call  
     * @param args : Array of the method parameters
     * @return : The call result of the method given in parameter 
     * @throws NoSuchFieldException
     */
    public Object callPetalsService(Method method, Object[] args) throws Throwable, JbiClientException {
        Object result = null;
        try {
            Document messageContent = null;
            if ((this.service.getOperationByName(method.getName())) != null)
                messageContent = service.getServiceMarshaller().marshallMethodParam(this.service.getOperationByName(method.getName()) ,args);
            else
                throw new JbiClientException((" There is no operation in the wsdl service matching the method name : " + (method.getName())));
            
            ServiceEndpoint[] svcEndpoints = componentContext.getEndpointsForService(service.getServiceName());
            MessageExchangeFactory ef = deliveryChannel.createExchangeFactory();
            MessageExchange me = null;
            if (method.getReturnType().equals(void.class))
                me = ef.createInOnlyExchange();
            else
                me = ef.createInOutExchange();
            
            me.setEndpoint(svcEndpoints[0]);
            Operation serviceOperation = service.getOperationByName(method.getName());
            QName operation = null;
            if (((serviceOperation.getSoapActionName()) == null) || (serviceOperation.getSoapActionName().equals(""))) {
                operation = new QName(serviceName.getNamespaceURI() , serviceOperation.getOperationName());
            } else
                operation = QName.valueOf(serviceOperation.getSoapActionName());
            
            me.setOperation(operation);
            NormalizedMessage nm = me.createMessage();
            nm.setContent(new DOMSource(messageContent));
            if (me instanceof InOut)
                ((InOut)(me)).setInMessage(nm);
            else
                ((InOnly)(me)).setInMessage(nm);
            
            deliveryChannel.sendSync(me);
            if (me instanceof InOut) {
                NormalizedMessage nmr = me.getMessage("OUT");
                if ((nmr != null) && ((nmr.getContent()) != null)) {
                    result = service.getServiceUnmarshaller().unmarshallMethodResult(service.getOperationByName(method.getName()) ,nmr.getContent());
                } else if (((me.getFault()) != null) && ((me.getFault().getContent()) != null)) {
                    Exception exception = ((Exception)(service.getServiceUnmarshaller().unmarshallException(service.getOperationByName(method.getName()) ,me.getFault().getContent())));
                    throw exception;
                } else
                    throw new JbiClientException(" the out message has nether content nor fault ");
                
            } 
        } catch (DataBindingException e) {
            e.printStackTrace();
            throw new JbiClientException(((" Error occurs during the call of the JBI endpoint for the " + (service.getServiceName())) != null ? service.getServiceName().toString() : null) , e);
        }
        return result;
    }
    
}

