package org.ow2.petals.se.sca.listener;

import org.ow2.petals.component.framework.listener.AbstractJBIListener;
import org.ow2.petals.component.framework.listener.AbstractListener;
import org.ow2.petals.databinding.jaxb.DataBindingException;
import org.w3c.dom.Document;
import org.ow2.petals.component.framework.api.message.Exchange;
import javax.jbi.messaging.Fault;
import java.lang.reflect.InvocationTargetException;
import java.util.logging.Logger;
import java.util.Map;
import javax.jbi.messaging.MessagingException;
import java.lang.reflect.Method;
import org.ow2.petals.databinding.jaxb.service.Operation;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import javax.xml.namespace.QName;
import org.ow2.petals.se.sca.model.SCAComposite;
import org.ow2.petals.se.sca.SCAServiceEngine;
import org.ow2.petals.databinding.jaxb.service.Service;
import org.ow2.petals.databinding.jaxb.serialization.ServiceMarshaller;
import org.ow2.petals.databinding.jaxb.serialization.ServiceUnmarshaller;
import javax.xml.transform.Source;

public class JbiListener extends AbstractJBIListener {
    private Logger logger;
    
    @Override
    protected void init() {
        this.logger = getLogger();
    }
    
    @Override
    protected void onJBIMessage(Exchange exchange) {
        QName operation = exchange.getOperation();
        Provides provides = getProvides();
        SCAServiceEngine scaServiceEngine = ((SCAServiceEngine)(this.component));
        String serviceUnitName = scaServiceEngine.getServiceUnitNamePool().get(provides);
        SCAComposite scaComposite = scaServiceEngine.getScaCompositePool().get(serviceUnitName);
        Service service = scaComposite.getCompositeServices().get(provides);
        if (service == null) {
            handleError(("Can\'t find the databinding service to " + "marshall/unmarshall the message content") ,exchange);
            return ;
        } 
        Operation serviceOperation = service.getOperationByName(operation.getLocalPart());
        if (serviceOperation == null) {
            handleError(("Can\'t find the SCA service operation matching " + "with the message exchange operation") ,exchange);
            return ;
        } 
        Object[] methodParams;
        try {
            methodParams = service.getServiceUnmarshaller().unmarshallMethodParam(serviceOperation ,exchange.getInMessageContentAsSource());
        } catch (MessagingException e) {
            handleError("Can\'t get the messge content from the message exchange" ,exchange);
            return ;
        } catch (DataBindingException e) {
            handleError("Can\'t unmarshall the message content" ,exchange);
            return ;
        }
        Object compositeInterface = scaComposite.getServicesInsances().get(provides);
        if (compositeInterface == null) {
            handleError(("Can\'t get the service implementation from the" + "SCA composite") ,exchange);
            return ;
        } 
        Method methodToInvoke;
        try {
            ClassLoader threadClassLoader = Thread.currentThread().getContextClassLoader();
            methodToInvoke = compositeInterface.getClass().getDeclaredMethod(serviceOperation.getOperationName() ,serviceOperation.getParameterTypes());
            Object object = null;
            try {
                Thread.currentThread().setContextClassLoader(compositeInterface.getClass().getClassLoader());
                object = methodToInvoke.invoke(compositeInterface ,methodParams);
                Thread.currentThread().setContextClassLoader(threadClassLoader);
            } catch (InvocationTargetException e) {
                if (isInstanceOf(e ,serviceOperation.getExceptionType())) {
                    Document exception = service.getServiceMarshaller().marshallMethodException(serviceOperation ,((Exception)(e.getCause())));
                    Fault fault = exchange.createFault();
                    fault.setContent(org.ow2.petals.component.framework.util.SourceHelper.createSource(exception));
                    exchange.setFault(fault);
                } else {
                    e.printStackTrace();
                    logger.warning((("Technical exception is throwed when calling" + "the service operation : ") + e));
                    handleError(("Technical exception is throwed when calling" + "the service operation : ") ,exchange);
                }
                return ;
            }
            if ((exchange.isInOutPattern()) || (exchange.isInOptionalOutPattern())) {
                Document response = service.getServiceMarshaller().marshallMethodResult(serviceOperation ,new Object[]{ object });
                exchange.setOutMessageContent(response);
            } 
        } catch (Exception e) {
            e.printStackTrace();
            handleError("Error occurs while calling the service operation" ,exchange);
        }
    }
    
    /** 
     * Handle an error
     * 
     * @param message
     * @param exchange
     */
    private void handleError(String message, Exchange exchange) {
        handleError(new Exception(message) ,exchange);
    }
    
    /** 
     * Handle an error
     * 
     * @param exception
     * @param Exchange
     */
    private void handleError(Exception exception, Exchange exchange) {
        logger.warning(("ERROR : " + (exception.getMessage())));
        exchange.setError(new Exception("Error : The SCA service execution generated an exception" , exception));
    }
    
    /** 
     * 
     * @param e
     * @param exceptionTypes
     * @return
     */
    private boolean isInstanceOf(Throwable e, Class<?> [] exceptionTypes) {
        for (Class<?>  exceptionType : exceptionTypes) {
            if (e.getClass().equals(exceptionType))
                return true;
            
        }
        return false;
    }
    
}

