package org.ow2.petals.se.sca.su;

import java.util.ArrayList;
import org.objectweb.fractal.api.Component;
import javax.jbi.component.ComponentContext;
import org.eclipse.stp.sca.Composite;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.eclipse.emf.common.util.EList;
import org.w3c.dom.Element;
import java.io.File;
import org.ow2.petals.se.sca.constants.FrascatiConstants;
import org.eclipse.stp.sca.Interface;
import org.eclipse.stp.sca.JavaInterface;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import java.util.List;
import java.util.Map;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.se.sca.factory.PetalsFactory;
import java.util.Properties;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.se.sca.constants.ProvidesConstants;
import javax.xml.namespace.QName;
import org.ow2.petals.se.sca.model.SCAComposite;
import org.ow2.petals.se.sca.SCAServiceEngine;
import org.ow2.petals.databinding.jaxb.service.Service;
import org.ow2.petals.component.framework.api.su.ServiceUnitListener;
import org.ow2.petals.component.framework.jbidescriptor.generated.Services;
import java.net.URL;
import java.net.URLClassLoader;

/** 
 * 
 * @author Mohammed EL JAI
 */
public class FrascatiSUListner implements ServiceUnitListener {
    /** 
     *
     */
    private SCAServiceEngine component;
    
    /** 
     * 
     * @param scaServiceEngine
     */
    public FrascatiSUListner(SCAServiceEngine scaServiceEngine) {
        super();
        this.component = scaServiceEngine;
    }
    
    public void onSUDeployed(String serviceUnitName, String suRootPath, Jbi descriptor) throws PEtALSCDKException {
        String oldFractalProvider = java.lang.System.getProperty(FrascatiConstants.FRACTAL_PROVIDER);
        String oldFractalProviders = java.lang.System.getProperty(FrascatiConstants.FRACTAL_PROVIDERS);
        ClassLoader oldThreadClassLoader = ((ClassLoader)(Thread.currentThread().getContextClassLoader()));
        Component scaDomain = null;
        SCAComposite scaComposite = null;
        try {
            scaComposite = new SCAComposite();
            URL[] seClassPath = org.ow2.petals.component.framework.util.ClassLoaderUtil.getUrls(this.component.getContext().getInstallRoot());
            URL[] arrayOfSuJar = org.ow2.petals.component.framework.util.ClassLoaderUtil.getUrls(suRootPath);
            URL[] bootStrapCompositeJars = createBootStrapCompositeJars(arrayOfSuJar);
            ClassLoader bootStrapClassLoader = new URLClassLoader(bootStrapCompositeJars , getClass().getClassLoader());
            Thread.currentThread().setContextClassLoader(bootStrapClassLoader);
            for (Provides provides : getProvidesList(descriptor)) {
                this.component.getServiceUnitNamePool().put(provides ,serviceUnitName);
                ConfigurationExtensions extensions = new ConfigurationExtensions(provides.getAny());
                String compositeFileName = extensions.get(ProvidesConstants.ATTRIBUTE_COMPOSITE_FILE);
                if (!(compositeFileName.endsWith(FrascatiConstants.COMPOSITE_FILE_EXTENSION)))
                    compositeFileName = compositeFileName + (FrascatiConstants.COMPOSITE_FILE_EXTENSION);
                
                String compositeFilepath = suRootPath + compositeFileName;
                Composite composite = null;
                if (new File(compositeFilepath).exists())
                    composite = org.ow2.petals.se.sca.Util.parse(compositeFilepath);
                else
                    composite = org.ow2.petals.se.sca.Util.parse(compositeFileName);
                
                SCAServiceEngine frascatiComponent = ((SCAServiceEngine)(this.component));
                org.ow2.petals.se.sca.binding.BindingProcess.processBinding(composite ,bootStrapClassLoader ,provides ,suRootPath ,frascatiComponent.getContext() ,provides.getWsdl() ,seClassPath ,scaComposite.getCompositeServices() ,scaComposite.getListOfReference());
                scaComposite.setCompositeClassLoader(bootStrapClassLoader);
                scaDomain = createTinfiComposite(suRootPath ,serviceUnitName ,bootStrapClassLoader ,seClassPath ,bootStrapCompositeJars ,compositeFilepath);
                scaComposite.setTinfiComponent(scaDomain);
                ClassLoader frascatiClassLoader = Thread.currentThread().getContextClassLoader();
                org.eclipse.stp.sca.Service service = composite.getService().get(0);
                JavaInterface serviceInterface = ((JavaInterface)(service.getInterface()));
                String interfaceName = serviceInterface.getInterface();
                Class<?>  serviceClass = frascatiClassLoader.loadClass(interfaceName);
                Object compositeInterfcae = org.ow2.frascati.tinfi.TinfiDomain.getService(scaDomain ,serviceClass ,service.getName());
                scaComposite.getServicesInsances().put(provides ,compositeInterfcae);
            }
            this.component.getScaCompositePool().put(serviceUnitName ,scaComposite);
        } catch (Exception e) {
            for (QName refernceName : scaComposite.getListOfReference())
                org.ow2.petals.se.sca.PetalsGateway.getSuReferencesPool().remove(refernceName);
            this.component.getScaCompositePool().remove(serviceUnitName);
            for (Provides provides : getProvidesList(descriptor))
                this.component.getServiceUnitNamePool().remove(provides);
            throw new PEtALSCDKException(e.getMessage() , e);
        } finally {
            if (oldFractalProvider != null)
                java.lang.System.setProperty(FrascatiConstants.FRACTAL_PROVIDER ,oldFractalProvider);
            
            if (oldFractalProviders != null)
                java.lang.System.setProperty(FrascatiConstants.FRACTAL_PROVIDERS ,oldFractalProviders);
            
            Thread.currentThread().setContextClassLoader(oldThreadClassLoader);
        }
    }
    
    /** 
     * 
     * @param arrayOfSuJar
     * @return
     */
    private URL[] createBootStrapCompositeJars(URL[] arrayOfSuJar) {
        int arraySize = arrayOfSuJar.length;
        List<java.net.URL>  listOfJar = new ArrayList<java.net.URL> ();
        for (URL url : arrayOfSuJar) {
            if (!(url.getPath().contains("jaxb"))) {
                listOfJar.add(url);
            } else
                arraySize--;
            
        }
        URL[] bootStrapCompositeJars = new URL[arraySize];
        return listOfJar.toArray(bootStrapCompositeJars);
    }
    
    public void onSUUndeployed(String serviceUnitName, String suRootPath, Jbi descriptor) throws PEtALSCDKException {
        SCAComposite scaComposite = this.component.getScaCompositePool().get(serviceUnitName);
        if (scaComposite != null) {
            for (QName refernceName : scaComposite.getListOfReference())
                org.ow2.petals.se.sca.PetalsGateway.getSuReferencesPool().remove(refernceName);
            this.component.getScaCompositePool().remove(serviceUnitName);
        } 
        for (Provides provides : getProvidesList(descriptor))
            this.component.getServiceUnitNamePool().remove(provides);
    }
    
    public void onSUStarted(String serviceUnitName) throws PEtALSCDKException {
    }
    
    public void onSUStopped(String serviceUnitName) throws PEtALSCDKException {
    }
    
    /** 
     * 
     * @param suRootPath
     * @param serviceUnitName
     * @param bootStrapClassLoader
     * @param seClassPath
     * @param bootStrapCompositeJars
     * @param newCompositeFilePath
     * @return
     */
    private Component createTinfiComposite(String suRootPath, String serviceUnitName, ClassLoader bootStrapClassLoader, URL[] seClassPath, URL[] bootStrapCompositeJars, String newCompositeFilePath) {
        Properties saveProps = ((Properties)(java.lang.System.getProperties().clone()));
        String juliacBaseDir = (suRootPath + serviceUnitName) + "-juliac";
        new File(juliacBaseDir).mkdir();
        PetalsFactory factory = new PetalsFactory();
        factory.setClassLoader(((URLClassLoader)(bootStrapClassLoader)));
        factory.setOutput(juliacBaseDir);
        URL[] libs = new URL[(bootStrapCompositeJars.length) + (seClassPath.length)];
        for (int i = 0 ; i < (bootStrapCompositeJars.length) ; i++) {
            libs[i] = bootStrapCompositeJars[i];
        }
        for (int i = 0 ; i < (seClassPath.length) ; i++) {
            libs[(i + (bootStrapCompositeJars.length))] = seClassPath[i];
        }
        Component scaDomain = factory.getComposite(newCompositeFilePath ,libs);
        factory = null;
        java.lang.System.setProperties(saveProps);
        Runtime.getRuntime().gc();
        return scaDomain;
    }
    
    /** 
     * 
     * @param descriptor
     * @return
     */
    private List<org.ow2.petals.component.framework.jbidescriptor.generated.Provides>  getProvidesList(Jbi descriptor) {
        List<org.ow2.petals.component.framework.jbidescriptor.generated.Provides>  providesList = new ArrayList<org.ow2.petals.component.framework.jbidescriptor.generated.Provides> ();
        Services services = descriptor.getServices();
        if (services != null) {
            providesList = services.getProvides();
        } 
        return providesList;
    }
    
}

